@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: downloader-view.c,v 1.27 2003/10/19 18:16:54 marco Exp $
+ *  $Id: downloader-view.c,v 1.42 2004/01/12 19:59:08 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,30 +24,26 @@
 
 #include "downloader-view.h"
 #include "ephy-gui.h"
-#include "ephy-prefs.h"
 #include "ephy-ellipsizing-label.h"
-#include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
 #include "ephy-embed-shell.h"
 #include "ephy-cell-renderer-progress.h"
 #include "ephy-stock-icons.h"
 
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <eggstatusicon.h>
+#include <eggtraymanager.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtktreeviewcolumn.h>
-#include <bonobo/bonobo-i18n.h>
-#include <libgnomeui/gnome-dialog-util.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <gtk/gtkprogressbar.h>
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <glib/gi18n.h>
 
 #define CONF_DOWNLOADING_SHOW_DETAILS "/apps/epiphany/dialogs/downloader_show_details"
 
 enum
 {
 	COL_PERCENT,
-	COL_FILENAME,
-	COL_SIZE,
+	COL_FILE,
 	COL_REMAINING,
 	COL_DOWNLOAD_OBJECT
 };
@@ -57,22 +53,23 @@ enum
 struct DownloaderViewPrivate
 {
 	GtkTreeModel *model;
-	GHashTable *details_hash;
+	GHashTable *downloads_hash;
 
 	/* Widgets */
 	GtkWidget *window;
 	GtkWidget *treeview;
-
-	GtkWidget *open_button;
 	GtkWidget *pause_button;
 	GtkWidget *abort_button;
+
+	EggStatusIcon *status_icon;
+
+	long remaining_secs;
 };
 
 typedef struct
 {
 	gboolean is_paused;
 	gboolean can_abort;
-	gboolean can_open;
 	DownloaderViewPrivate *priv;
 } ControlsInfo;
 
@@ -80,22 +77,19 @@ enum
 {
 	PROP_WINDOW,
 	PROP_TREEVIEW,
-	PROP_OPEN_BUTTON,
 	PROP_PAUSE_BUTTON,
 	PROP_ABORT_BUTTON,
-	PROP_DETAILS_BUTTON
 };
 
 static const
 EphyDialogProperty properties [] =
 {
-	{ PROP_WINDOW, "download_manager_dialog", NULL, PT_NORMAL, NULL},
-        { PROP_TREEVIEW, "clist", NULL, PT_NORMAL, NULL },
-	{ PROP_OPEN_BUTTON, "open_button", NULL, PT_NORMAL, NULL },
-	{ PROP_PAUSE_BUTTON, "pause_button", NULL, PT_NORMAL, NULL },
-	{ PROP_ABORT_BUTTON, "abort_button", NULL, PT_NORMAL, NULL },
+	{ "download_manager_dialog",	NULL, PT_NORMAL, 0 },
+        { "clist",			NULL, PT_NORMAL, 0 },
+	{ "pause_button",		NULL, PT_NORMAL, 0 },
+	{ "abort_button",		NULL, PT_NORMAL, 0 },
 
-        { -1, NULL, NULL }
+        { NULL }
 };
 
 static void
@@ -115,9 +109,6 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv);
 gboolean
 download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
 			   DownloaderView *dv);
-void
-download_dialog_open_cb (GtkWidget *button,
-			 DownloaderView *dv);
 
 static GObjectClass *parent_class = NULL;
 
@@ -161,17 +152,38 @@ downloader_view_class_init (DownloaderViewClass *klass)
 	g_type_class_add_private (object_class, sizeof(DownloaderViewPrivate));
 }
 
+static void
+status_icon_activated (EggStatusIcon *icon, DownloaderView *dv)
+{
+	gtk_window_present (GTK_WINDOW (dv->priv->window));
+}
+
+static void
+show_status_icon (DownloaderView *dv)
+{
+	GdkPixbuf *pixbuf;
+
+	pixbuf = gdk_pixbuf_new_from_file (ephy_file ("epiphany-download.png"), NULL);
+	dv->priv->status_icon = egg_status_icon_new_from_pixbuf (pixbuf);
+	g_object_unref (pixbuf);
+
+	g_signal_connect (dv->priv->status_icon, "activate",
+			  G_CALLBACK (status_icon_activated), dv);
+}
+
 static void
 downloader_view_init (DownloaderView *dv)
 {
         dv->priv = EPHY_DOWNLOADER_VIEW_GET_PRIVATE (dv);
 
-	dv->priv->details_hash = g_hash_table_new_full
+	dv->priv->downloads_hash = g_hash_table_new_full
 		(g_direct_hash, g_direct_equal, NULL,
 		 (GDestroyNotify)gtk_tree_row_reference_free);
 
 	downloader_view_build_ui (dv);
 
+	show_status_icon (dv);
+
 	g_object_ref (embed_shell);
 }
 
@@ -179,8 +191,11 @@ static void
 downloader_view_finalize (GObject *object)
 {
 	DownloaderView *dv = EPHY_DOWNLOADER_VIEW (object);
-	g_hash_table_destroy (dv->priv->details_hash);
+
+	g_object_unref (dv->priv->status_icon);
+	g_hash_table_destroy (dv->priv->downloads_hash);
 	g_object_unref (embed_shell);
+
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -214,28 +229,24 @@ format_interval (long interval)
 static GtkTreeRowReference *
 get_row_from_download (DownloaderView *dv, EphyDownload *download)
 {
-	return  g_hash_table_lookup (dv->priv->details_hash, download);
+	return  g_hash_table_lookup (dv->priv->downloads_hash, download);
 }
 
 static void
-download_changed_cb (EphyDownload *download, DownloaderView *dv)
+update_download_row (DownloaderView *dv, EphyDownload *download)
 {
 	GtkTreeRowReference *row_ref;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	EphyDownloadState state;
-	int percent;
-	long total;
-	long remaining_secs;
-	char *remaining;
-	char *size;
+	long total, current, remaining_secs;
+	char *remaining, *file, *cur_progress;
 	struct tm;
+	int percent;
 
 	row_ref = get_row_from_download (dv, download);
 	g_return_if_fail (row_ref != NULL);
 
-	total =  ephy_download_get_total_progress (download) / 1024;
-
 	/* State special casing */
 	state = ephy_download_get_state (download);
 	switch (state)
@@ -245,9 +256,8 @@ download_changed_cb (EphyDownload *download, DownloaderView *dv)
 		remaining_secs = 0;	
 		break;
 	case EPHY_DOWNLOAD_COMPLETED:
-		percent = 100;
-		remaining_secs = 0;
-		break;
+		downloader_view_remove_download (dv, download);
+		return;
 	case EPHY_DOWNLOAD_DOWNLOADING:
 	case EPHY_DOWNLOAD_PAUSED:	
 		percent = ephy_download_get_percent (download);
@@ -259,23 +269,93 @@ download_changed_cb (EphyDownload *download, DownloaderView *dv)
 		break;
 	}
 
-	size = g_strdup_printf ("%ld kB", total);
-	remaining = format_interval (remaining_secs);
+	total = ephy_download_get_total_progress (download);
+	current = ephy_download_get_current_progress (download);
+
+	cur_progress = gnome_vfs_format_file_size_for_display (current);
+
+	if (total != -1)
+	{
+		char *total_progress;
+
+		total_progress = gnome_vfs_format_file_size_for_display (total);
+		file = g_strdup_printf ("%s\n%s of %s",
+					ephy_download_get_name (download),
+					cur_progress, total_progress);
+		g_free (total_progress);
+	}
+	else
+	{
+		file = g_strdup_printf ("%s\n%s",
+					ephy_download_get_name (download),
+					cur_progress);
+	}
+
+	if (remaining_secs < 0)
+	{
+		remaining = g_strdup (_("Unknown"));
+	}
+	else
+	{
+		remaining = format_interval (remaining_secs);
+	}
 
 	path = gtk_tree_row_reference_get_path (row_ref);
 	gtk_tree_model_get_iter (dv->priv->model, &iter, path);
 	gtk_list_store_set (GTK_LIST_STORE (dv->priv->model),
 			    &iter,
 			    COL_PERCENT, percent,
-			    COL_SIZE, size,
+			    COL_FILE, file,
 			    COL_REMAINING, remaining,
 			    -1);
 	gtk_tree_path_free (path);
 
-	g_free (size);
+	g_free (file);
 	g_free (remaining);
 }
 
+static void
+seconds_remaining_total (EphyDownload *download, gpointer data, DownloaderView *dv)
+{
+	long secs;
+
+	secs = ephy_download_get_remaining_time (download);
+	if (secs > 0)
+	{
+		dv->priv->remaining_secs += secs;
+	}
+}
+
+static void
+update_status_icon (DownloaderView *dv)
+{
+	char *tooltip, *remaining;
+	int downloads;
+
+	dv->priv->remaining_secs = 0;
+	g_hash_table_foreach (dv->priv->downloads_hash,
+			      (GHFunc) seconds_remaining_total, dv);
+	
+	remaining = format_interval (dv->priv->remaining_secs);
+	downloads = g_hash_table_size (dv->priv->downloads_hash);
+	tooltip = g_strdup_printf ("%d downloads, with a total of "
+				   "%s minutes remaining",
+				   downloads, remaining);
+
+	egg_status_icon_set_tooltip (dv->priv->status_icon,
+				     tooltip, NULL);
+
+	g_free (tooltip);
+	g_free (remaining);
+}
+
+static void
+download_changed_cb (EphyDownload *download, DownloaderView *dv)
+{
+	update_download_row (dv, download);
+	update_status_icon (dv);
+}
+
 void
 downloader_view_add_download (DownloaderView *dv,
 			      EphyDownload *download)
@@ -284,35 +364,30 @@ downloader_view_add_download (DownloaderView *dv,
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
 	GtkTreePath *path;
-	char *name;
-
 
 	gtk_list_store_append (GTK_LIST_STORE (dv->priv->model),
 			       &iter);
+	gtk_list_store_set (GTK_LIST_STORE (dv->priv->model),
+			    &iter, COL_DOWNLOAD_OBJECT, download, -1);
 
 	path =  gtk_tree_model_get_path (GTK_TREE_MODEL (dv->priv->model), &iter);
 	row_ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (dv->priv->model), path);
 	gtk_tree_path_free (path);
 
-	g_hash_table_insert (dv->priv->details_hash,
+	g_hash_table_insert (dv->priv->downloads_hash,
 			     download,
 			     row_ref);
 
-	name = ephy_download_get_name (download);
-	gtk_list_store_set (GTK_LIST_STORE (dv->priv->model),
-			    &iter,
-			    COL_FILENAME, name,
-			    COL_DOWNLOAD_OBJECT, download,
-			    -1);
-	g_free (name);
+	update_download_row (dv, download);
+	update_status_icon (dv);
 
 	selection = gtk_tree_view_get_selection
 		(GTK_TREE_VIEW(dv->priv->treeview));
 	gtk_tree_selection_unselect_all (selection);
 	gtk_tree_selection_select_iter (selection, &iter);
 
-	g_signal_connect (download, "changed",
-			  G_CALLBACK (download_changed_cb), dv);
+	g_signal_connect_object (download, "changed",
+				 G_CALLBACK (download_changed_cb), dv, 0);
 
 	ephy_dialog_show (EPHY_DIALOG (dv));
 }
@@ -334,20 +409,18 @@ downloader_view_build_ui (DownloaderView *dv)
                                "download_manager_dialog");
 
 	/* lookup needed widgets */
-	priv->window = ephy_dialog_get_control(d, PROP_WINDOW);
-	priv->treeview = ephy_dialog_get_control (d, PROP_TREEVIEW);
-	priv->open_button = ephy_dialog_get_control (d, PROP_OPEN_BUTTON);
-	priv->pause_button = ephy_dialog_get_control (d, PROP_PAUSE_BUTTON);
-	priv->abort_button = ephy_dialog_get_control (d, PROP_ABORT_BUTTON);
+	priv->window = ephy_dialog_get_control(d, properties[PROP_WINDOW].id);
+	priv->treeview = ephy_dialog_get_control (d, properties[PROP_TREEVIEW].id);
+	priv->pause_button = ephy_dialog_get_control (d, properties[PROP_PAUSE_BUTTON].id);
+	priv->abort_button = ephy_dialog_get_control (d, properties[PROP_ABORT_BUTTON].id);
 
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)),
 				     GTK_SELECTION_SINGLE);
 
-	liststore = gtk_list_store_new (5,
+	liststore = gtk_list_store_new (4,
 					G_TYPE_INT,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
-					G_TYPE_STRING,
 					G_TYPE_OBJECT);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW(priv->treeview),
@@ -363,33 +436,23 @@ downloader_view_build_ui (DownloaderView *dv)
 						     "value", 0,
 						     NULL);
 	column = gtk_tree_view_get_column (GTK_TREE_VIEW(priv->treeview), 0);
-        gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
         gtk_tree_view_column_set_sort_column_id (column, COL_PERCENT);
 
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(priv->treeview),
-						     COL_FILENAME, _("Filename"),
+						     COL_FILE, _("File"),
 						     renderer,
-						     "text", COL_FILENAME,
+						     "text", COL_FILE,
 						     NULL);
 	column = gtk_tree_view_get_column (GTK_TREE_VIEW(priv->treeview),
-					   COL_FILENAME);
-        gtk_tree_view_column_set_resizable (column, TRUE);
+					   COL_FILE);
+        gtk_tree_view_column_set_expand (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
-        gtk_tree_view_column_set_sort_column_id (column, COL_FILENAME);
-
-	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(priv->treeview),
-						     COL_SIZE, _("Size"),
-						     renderer,
-						     "text", COL_SIZE,
-						     NULL);
-	column = gtk_tree_view_get_column (GTK_TREE_VIEW(priv->treeview),
-					   COL_SIZE);
-        gtk_tree_view_column_set_resizable (column, TRUE);
-        gtk_tree_view_column_set_reorderable (column, TRUE);
-        gtk_tree_view_column_set_sort_column_id (column, COL_SIZE);
+        gtk_tree_view_column_set_sort_column_id (column, COL_FILE);
 
+	renderer = gtk_cell_renderer_text_new ();
+	g_object_set (renderer, "xalign", 0.5, NULL);
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW(priv->treeview),
 						     COL_REMAINING, _("Remaining"),
 						     renderer,
@@ -397,7 +460,6 @@ downloader_view_build_ui (DownloaderView *dv)
 						     NULL);
 	column = gtk_tree_view_get_column (GTK_TREE_VIEW(priv->treeview),
 					   COL_REMAINING);
-        gtk_tree_view_column_set_resizable (column, TRUE);
         gtk_tree_view_column_set_reorderable (column, TRUE);
         gtk_tree_view_column_set_sort_column_id (column, COL_REMAINING);
 
@@ -436,10 +498,12 @@ download_dialog_pause_cb (GtkButton *button, DownloaderView *dv)
 	if (state == EPHY_DOWNLOAD_DOWNLOADING)
 	{
 		ephy_download_pause (download);
+		gtk_button_set_label (GTK_BUTTON (dv->priv->pause_button), _("_Resume"));
 	}
 	else if (state == EPHY_DOWNLOAD_PAUSED)
 	{
 		ephy_download_resume (download);
+		gtk_button_set_label (GTK_BUTTON (dv->priv->pause_button), _("_Pause"));
 	}
 }
 
@@ -448,24 +512,60 @@ downloader_view_remove_download (DownloaderView *dv, EphyDownload *download)
 {
 	GtkTreeRowReference *row_ref;
 	GtkTreePath *path = NULL;
-	GtkTreeIter iter;
+	GtkTreeIter iter, iter2;
 
 	row_ref = get_row_from_download (dv, download);
 	g_return_if_fail (row_ref);
 
+	/* Get the row we'll select after removal ("smart" selection) */
+	
 	path = gtk_tree_row_reference_get_path (row_ref);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model),
 				 &iter, path);
-
-	gtk_list_store_remove (GTK_LIST_STORE (dv->priv->model), &iter);
-
-	/* FIXME: smart selection */
-
-	g_hash_table_remove (dv->priv->details_hash,
-			     download);
-
 	gtk_tree_path_free (path);
 
+	row_ref = NULL;
+        iter2 = iter;			 
+	if (gtk_tree_model_iter_next (GTK_TREE_MODEL (dv->priv->model), &iter))
+        {
+        	path = gtk_tree_model_get_path  (GTK_TREE_MODEL (dv->priv->model), &iter);
+        	row_ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (dv->priv->model), path);
+        }
+        else
+        {
+        	path = gtk_tree_model_get_path (GTK_TREE_MODEL (dv->priv->model), &iter2);
+        	if (gtk_tree_path_prev (path))
+        	{
+        		row_ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (dv->priv->model),
+							      path);
+		}
+	}
+	gtk_tree_path_free (path);
+
+	/* Removal */
+	
+	gtk_list_store_remove (GTK_LIST_STORE (dv->priv->model), &iter2);
+	g_hash_table_remove (dv->priv->downloads_hash,
+			     download);
+
+	/* Actual selection */
+
+	if (row_ref != NULL)
+	{
+		path = gtk_tree_row_reference_get_path (row_ref);
+		if (path != NULL)
+		{
+			gtk_tree_view_set_cursor (GTK_TREE_VIEW (dv->priv->treeview),
+					          path, NULL, FALSE);
+			gtk_tree_path_free (path);
+		}
+		gtk_tree_row_reference_free (row_ref);
+	}
+
+	/* Close the dialog if there are no more downloads */
+
+	if (!g_hash_table_size (dv->priv->downloads_hash))
+		g_object_unref (G_OBJECT (dv));
 }
 
 void
@@ -494,25 +594,10 @@ gboolean
 download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
 			   DownloaderView *dv)
 {
-	return FALSE;
-}
+	if (egg_tray_manager_check_running (gdk_screen_get_default ()))
+	{
+		gtk_widget_hide (dv->priv->window);
+	}
 
-static void
-open_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
-			DownloaderView *dv)
-{
-}
-
-void
-download_dialog_open_cb (GtkWidget *button,
-			 DownloaderView *dv)
-{
-	GtkTreeSelection *selection;
-
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(dv->priv->treeview));
-
-	gtk_tree_selection_selected_foreach
-		(selection,
-		 (GtkTreeSelectionForeachFunc)open_selection_foreach,
-		 (gpointer)dv);
+	return TRUE;
 }
