@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2003 Marco Pesenti Gritti, Xan Lopez
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ContentHandler.h,v 1.8 2004/01/10 10:49:31 chpe Exp $
  */
 
 #ifndef __ContentHandler_h
@@ -31,17 +33,13 @@
 #include "nsIURI.h"
 #include "nsILocalFile.h"
 
+#include "nsCRT.h"
+#include "nsWeakReference.h"
+
 #include "nsCOMPtr.h"
 #include "nsISupports.h"
 #include "nsError.h"
 
-typedef enum
-{
-        ACTION_NONE,
-        ACTION_SAVEFORHELPER,
-        ACTION_OBJECT_NOTIFY
-} DownloadAction;
-
 #define G_CONTENTHANDLER_CID			     \
 { /* 16072c4a-23a6-4996-9beb-9335c06bbeae */         \
     0x16072c4a,                                      \
@@ -63,10 +61,8 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 
 	NS_METHOD FindHelperApp (void);
 	NS_METHOD LaunchHelperApp (void);
-	NS_METHOD ShowHelperProgressDialog (void);
 
 	NS_METHOD GetLauncher (nsIHelperAppLauncher * *_retval);
-	NS_METHOD GetContext (nsISupports * *_retval);
 	NS_METHOD SetHelperApp(GnomeVFSMimeApplication *mHelperApp,
 			       PRBool alwaysUse);
 	NS_METHOD SynchroniseMIMEInfo (void);
@@ -78,10 +74,8 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	NS_METHOD MIMEAskAction (void);
 	
 	nsCOMPtr<nsIHelperAppLauncher> mLauncher;
-	nsCOMPtr<nsISupports> mContext;
 
 	nsCOMPtr<nsIURI> mUri;
-	PRInt64 mTimeDownloadStarted;
 	nsCOMPtr<nsIFile> mTempFile;
 
 	char *mMimeType;	
@@ -90,13 +84,6 @@ class GContentHandler : public nsIHelperAppLauncherDialog
 	
 	nsCString mUrl;
 	nsCString mScheme;
-
-	PRBool mDownloadCanceled;
-	PRBool mHelperProgress;
-	
-	nsCOMPtr<nsIWebProgressListener> mListener;
 };
 
-extern nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory);
-
 #endif
