@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.h,v 1.8 2003/12/09 22:23:06 marco Exp $
+ *  $Id: EphyBrowser.h,v 1.9 2004/01/04 12:52:05 chpe Exp $
  */
 
 #ifndef EPHY_BROWSER_H
@@ -119,6 +119,8 @@ public:
 	nsresult GetDocumentUrl (nsCString &url);
 	nsresult GetTargetDocumentUrl (nsCString &url);
 
+	nsresult GetHasModifiedForms (PRBool *modified);
+
 	nsCOMPtr<nsIWebBrowser> mWebBrowser;
 
 private:
@@ -135,6 +137,7 @@ private:
 	nsresult SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell);
 	nsresult GetSHistory (nsISHistory **aSHistory);
 	nsresult GetContentViewer (nsIContentViewer **aViewer);
+	nsresult GetDocumentHasModifiedForms (nsIDOMDocument *aDomDoc, PRUint32 *aNumTextFields, PRBool *aHasTextArea);
 };
 
 #endif
