@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-tabs-menu.c,v 1.22 2004/01/01 22:57:16 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -22,12 +24,14 @@
 
 #include "ephy-tabs-menu.h"
 #include "ephy-string.h"
-#include "egg-menu-merge.h"
 #include "ephy-marshal.h"
 #include "ephy-shell.h"
 #include "ephy-debug.h"
-#include "egg-action.h"
 
+#include <gtk/gtkaccelmap.h>
+#include <gtk/gtkaction.h>
+#include <gtk/gtkradioaction.h>
+#include <gtk/gtkuimanager.h>
 #include <string.h>
 #include <stdlib.h>
 #include <libxml/entities.h>
@@ -37,10 +41,13 @@
 /**
  * Private data
  */
+
+#define EPHY_TABS_MENU_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TABS_MENU, EphyTabsMenuPrivate))
+
 struct _EphyTabsMenuPrivate
 {
 	EphyWindow *window;
-	EggActionGroup *action_group;
+	GtkActionGroup *action_group;
 	guint ui_id;
 };
 
@@ -49,7 +56,6 @@ struct _EphyTabsMenuPrivate
  */
 static void	ephy_tabs_menu_class_init	(EphyTabsMenuClass *klass);
 static void	ephy_tabs_menu_init	  	(EphyTabsMenu *menu);
-static void	ephy_tabs_menu_finalize_impl 	(GObject *o);
 
 enum
 {
@@ -57,8 +63,6 @@ enum
 	PROP_WINDOW
 };
 
-static gpointer g_object_class;
-
 /**
  * EphyTabsMenu object
  */
@@ -91,6 +95,56 @@ ephy_tabs_menu_get_type (void)
         return ephy_tabs_menu_type;
 }
 
+static void
+tab_added_cb (EphyNotebook *notebook, EphyEmbed *embed, EphyTabsMenu *menu)
+{
+	EphyTab *tab;
+	GtkAction *action;
+
+        g_return_if_fail (EPHY_IS_EMBED (embed));
+	tab = ephy_tab_for_embed (embed);
+
+	action = GTK_ACTION (ephy_tab_get_action (tab));
+	gtk_action_group_add_action (menu->priv->action_group, action);
+
+	ephy_tabs_menu_update (menu);
+}
+
+static void
+tab_removed_cb (EphyNotebook *notebook, EphyEmbed *embed, EphyTabsMenu *menu)
+{
+	EphyTab *tab;
+	GtkAction *action;
+                                                                                                                             
+        g_return_if_fail (EPHY_IS_EMBED (embed));
+	tab = ephy_tab_for_embed (embed);
+
+	action = GTK_ACTION (ephy_tab_get_action (tab));
+	gtk_action_group_remove_action (menu->priv->action_group, action);
+
+	ephy_tabs_menu_update (menu);
+}
+
+static void
+ephy_tabs_menu_set_window (EphyTabsMenu *menu, EphyWindow *window)
+{
+	GtkWidget *notebook;
+	GtkUIManager *merge;
+
+	menu->priv->window = window;
+
+	merge = GTK_UI_MANAGER (window->ui_merge);
+	menu->priv->action_group = gtk_action_group_new ("TabsActions");
+	gtk_ui_manager_insert_action_group (merge, menu->priv->action_group, 0);
+	g_object_unref (menu->priv->action_group);
+
+	notebook = ephy_window_get_notebook (window);
+	g_signal_connect_object (notebook, "tab_added",
+			         G_CALLBACK (tab_added_cb), menu, 0);
+	g_signal_connect_object (notebook, "tab_removed",
+			         G_CALLBACK (tab_removed_cb), menu, 0);
+}
+
 static void
 ephy_tabs_menu_set_property (GObject *object,
 			     guint prop_id,
@@ -102,7 +156,8 @@ ephy_tabs_menu_set_property (GObject *object,
         switch (prop_id)
         {
                 case PROP_WINDOW:
-                        m->priv->window = g_value_get_object (value);
+                        ephy_tabs_menu_set_window
+				(m, EPHY_WINDOW (g_value_get_object (value)));
                         break;
         }
 }
@@ -128,9 +183,6 @@ ephy_tabs_menu_class_init (EphyTabsMenuClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	G_OBJECT_CLASS (klass)->finalize = ephy_tabs_menu_finalize_impl;
-	g_object_class = g_type_class_peek_parent (klass);
-
 	object_class->set_property = ephy_tabs_menu_set_property;
 	object_class->get_property = ephy_tabs_menu_get_property;
 
@@ -139,19 +191,17 @@ ephy_tabs_menu_class_init (EphyTabsMenuClass *klass)
                                          g_param_spec_object ("EphyWindow",
                                                               "EphyWindow",
                                                               "Parent window",
-                                                              EPHY_WINDOW_TYPE,
+                                                              EPHY_TYPE_WINDOW,
                                                               G_PARAM_READWRITE |
 							      G_PARAM_CONSTRUCT_ONLY));
+
+	g_type_class_add_private (object_class, sizeof(EphyTabsMenuPrivate));
 }
 
 static void
 ephy_tabs_menu_init (EphyTabsMenu *menu)
 {
-	EphyTabsMenuPrivate *p;
-	
-	p = g_new0 (EphyTabsMenuPrivate, 1);
-	
-	menu->priv = p;
+	menu->priv = EPHY_TABS_MENU_GET_PRIVATE (menu);
 
 	menu->priv->ui_id = 0;
 	menu->priv->action_group = NULL;
@@ -161,42 +211,16 @@ static void
 ephy_tabs_menu_clean (EphyTabsMenu *menu)
 {
 	EphyTabsMenuPrivate *p = menu->priv;
-	EggMenuMerge *merge = EGG_MENU_MERGE (p->window->ui_merge);
+	GtkUIManager *merge = GTK_UI_MANAGER (p->window->ui_merge);
 
 	if (p->ui_id > 0)
 	{
-		egg_menu_merge_remove_ui (merge, p->ui_id);
-		egg_menu_merge_ensure_update (merge);
+		gtk_ui_manager_remove_ui (merge, p->ui_id);
+		gtk_ui_manager_ensure_update (merge);
 		p->ui_id = 0;
 	}
-
-	if (p->action_group != NULL)
-	{
-		egg_menu_merge_remove_action_group (merge, p->action_group);
-		g_object_unref (p->action_group);
-	}
 }
 
-static void
-ephy_tabs_menu_finalize_impl (GObject *o)
-{
-	EphyTabsMenu *menu = EPHY_TABS_MENU (o);
-	EphyTabsMenuPrivate *p = menu->priv;
-
-	if (p->action_group != NULL)
-	{
-		egg_menu_merge_remove_action_group
-			(EGG_MENU_MERGE (p->window->ui_merge),
-			 p->action_group);
-		g_object_unref (p->action_group);
-	}
-
-	g_free (p);
-
-	G_OBJECT_CLASS (g_object_class)->finalize (o);
-}
-
-
 EphyTabsMenu *
 ephy_tabs_menu_new (EphyWindow *window)
 {
@@ -205,25 +229,25 @@ ephy_tabs_menu_new (EphyWindow *window)
 					     NULL));
 }
 
-/* This code is from EggActionGroup:
- * Ideally either EggAction should support setting an accelerator from
- * a string or EggActionGroup would support adding single EggActionEntry's
- * to an action group.
- */
 static void
-tab_set_action_accelerator (EggActionGroup *action_group,
-			    EggAction *action,
+tab_set_action_accelerator (GtkActionGroup *action_group,
+			    GtkAction *action,
 			    guint tab_number)
 {
+	const char *action_name, *action_group_name;
 	char *accel_path = NULL;
 	char accel[7];
 	gint accel_number;
 	guint accel_key;
 	GdkModifierType accel_mods;
 
+	action_name = gtk_action_get_name (action);
+	action_group_name = gtk_action_group_get_name (action_group);
+
 	/* set the accel path for the menu item */
-	accel_path = g_strconcat ("<Actions>/", action_group->name, "/",
-				  action->name, NULL);
+	accel_path = g_strconcat ("<Actions>/", action_group_name, "/",
+				  action_name, NULL);
+	gtk_action_set_accel_path (action, accel_path);
 
 	/* Only the first ten tabs get accelerators starting from 1 through 0 */
 	if (tab_number < 10)
@@ -245,27 +269,22 @@ tab_set_action_accelerator (EggActionGroup *action_group,
 	{
 		gtk_accel_map_change_entry (accel_path, 0, 0, TRUE);
 	}
-
-	action->accel_quark = g_quark_from_string (accel_path);
-	g_free (accel_path);
 }
 
 void
 ephy_tabs_menu_update (EphyTabsMenu *menu)
 {
 	EphyTabsMenuPrivate *p;
-	EggMenuMerge *merge;
+	GtkUIManager *merge;
 	EphyTab *tab;
-	EggAction *action;
-	GString *xml;
+	GtkAction *action;
 	guint i = 0;
 	guint num = 0;
 	GList *tabs = NULL, *l;
-	GError *error = NULL;
 
 	g_return_if_fail (EPHY_IS_TABS_MENU (menu));
 	p = menu->priv;
-	merge = EGG_MENU_MERGE (p->window->ui_merge);
+	merge = GTK_UI_MANAGER (p->window->ui_merge);
 	
 	LOG ("Rebuilding open tabs menu")
 
@@ -278,39 +297,29 @@ ephy_tabs_menu_update (EphyTabsMenu *menu)
 	num = g_list_length (tabs);
 	if (num == 0) return;
 
-	p->action_group = egg_action_group_new ("TabsActions");
-
-	/* it's faster to preallocate, MIN is sanity check */
-	xml = g_string_sized_new (44 * MIN (num, 64) + 105);
-
-	g_string_append (xml, "<Root><menu><submenu name=\"TabsMenu\">"
-			      "<placeholder name=\"TabsOpen\">");
+	p->ui_id = gtk_ui_manager_new_merge_id (merge);
 
 	for (l = tabs; l != NULL; l = l->next)
 	{
+		const char *action_name;
+		char *name;
+
+
 		tab = (EphyTab *) l->data;
-		action = EGG_ACTION (ephy_tab_get_action (tab));
+		action = GTK_ACTION (ephy_tab_get_action (tab));
+		action_name = gtk_action_get_name (action);
+		name = g_strdup_printf ("%sMenu", action_name);
 
-		tab_set_action_accelerator (p->action_group, action, i);
+		tab_set_action_accelerator (p->action_group, action, i++);
 
-		egg_action_group_add_action (p->action_group, action);
-
-		g_string_append (xml, "<menuitem name=\"");
-		g_string_append (xml, action->name);
-		g_string_append (xml, "Menu\" verb=\"");
-		g_string_append (xml, action->name);
-		g_string_append (xml, "\"/>\n");
-
-		++i;
+		gtk_ui_manager_add_ui (merge, p->ui_id,
+				       "/menubar/TabsMenu/TabsOpen",
+				       name, action_name,
+				       GTK_UI_MANAGER_MENUITEM, FALSE);
+		g_free (name);
 	}
 
-	g_string_append (xml, "</placeholder></submenu></menu></Root>");
-
-	egg_menu_merge_insert_action_group (merge, p->action_group, 0);
-	p->ui_id = egg_menu_merge_add_ui_from_string
-				(merge, xml->str, -1, &error);	
-
-	g_string_free (xml, TRUE);
+	g_list_free (tabs);
 
 	STOP_PROFILER ("Rebuilding tabs menu")
 }
